%option c++
%option yyclass="WhileLexer"
%option noyywrap
%option nostdinit
%option yylineno
%option case-insensitive
%option full
%option prefix="WHILE_"

%{
    #include <iostream>
    #include <sstream>
    #include "parser.hpp"
    #include "whilelexer.h"
%}
%% // --------------------------------------------------------------------------
"//"[^\n]*\n                            {}
"/*"([^\*]*(\*+)[^*/])*([^\*]*)(\*+)"/" {}
[\n\t" "]*                              {}
";"                                     {return yy::WhileParser::token::SEMICOLON;}
"while"                                 {return yy::WhileParser::token::WHILE;}
"!="                                    {return yy::WhileParser::token::NEQ;}
"do"                                    {return yy::WhileParser::token::DO;}
"done"                                  {return yy::WhileParser::token::DONE;}
":="                                    {return yy::WhileParser::token::ASSIGN;}
"+"                                     {return yy::WhileParser::token::ADD;}
"-"                                     {return yy::WhileParser::token::SUB;}
[a-zA-Z][a-zA-Z0-9]*                    {
                                          yylval->stringval = new std::string(yytext);
                                          return yy::WhileParser::token::ID;
                                        }
[0-9]+                                  {
                                          yylval->intval = new int;
                                          std::stringstream s;
                                          s << yytext;
                                          s >> *(yylval->intval);
                                          return yy::WhileParser::token::NUM;
                                        }
.|\n                                    {
                                          std::cerr << "\n\nLEXICAL ERROR (Line "
                                            << yylineno << "): Unexpected character \""
                                            << yytext[0] << "\"\n";
                                        }
%% //---------------------------------------------------------------------------

WhileLexer::WhileLexer(std::istream& is)
    : WHILE_FlexLexer(&is)
{
}

int WhileLexer::yylex(yy::WhileParser::semantic_type* lval)
{
    yylval = lval;
    return yylex();
}

int WhileLexer::GetCurrentLineNumber()
{
    return yylineno;
}
